'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _editor = require('textcomplete/lib/editor');

var _editor2 = _interopRequireDefault(_editor);

var _utils = require('textcomplete/lib/utils');

var _lodash = require('lodash.bindall');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CALLBACK_METHODS = ['onKeydown', 'onKeyup'];

/**
 * @extends Editor
 * @prop {CodeMirror} cm
 */

var Codemirror = function (_Editor) {
  _inherits(Codemirror, _Editor);

  /**
   * @param {CodeMirror} cm
   */

  function Codemirror(cm) {
    _classCallCheck(this, Codemirror);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Codemirror).call(this));

    _this.cm = cm;

    (0, _lodash2.default)(_this, CALLBACK_METHODS);
    _this.startListening();
    return _this;
  }

  /** @override */


  _createClass(Codemirror, [{
    key: 'finalize',
    value: function finalize() {
      _get(Object.getPrototypeOf(Codemirror.prototype), 'finalize', this).call(this);
      this.stopListening();
      this.cm = null;
      return this;
    }

    /**
     * @override
     * @param {SearchResult} searchResult
     */

  }, {
    key: 'applySearchResult',
    value: function applySearchResult(searchResult) {
      var replace = searchResult.replace(this.getBeforeCursor(), this.getAfterCursor());
      if (Array.isArray(replace)) {
        this.cm.doc.setValue(replace[0] + replace[1]);
        var lines = replace[0].split('\n');
        this.cm.doc.setCursor(lines.length - 1, lines[lines.length - 1].length);
      }
      this.cm.focus();
    }

    /**
     * @override
     * @returns {{top: number, left: number}}
     */

  }, {
    key: 'getCursorOffset',
    value: function getCursorOffset() {
      var el = this.cm.display.cursorDiv.firstChild;
      var offset = (0, _utils.calculateElementOffset)(el);
      return { top: offset.top + parseInt(el.style.height, 10), left: offset.left };
    }

    /**
     * @override
     * @returns {string}
     */

  }, {
    key: 'getBeforeCursor',
    value: function getBeforeCursor() {
      var _getCursor = this.getCursor();

      var line = _getCursor.line;
      var ch = _getCursor.ch;

      var lines = this.getLines();
      var linesBeforeCursor = lines.slice(0, line);
      var currentLineBeforeCursor = lines[line].slice(0, ch);
      return linesBeforeCursor.concat([currentLineBeforeCursor]).join(this.lineSeparator());
    }

    /**
     * @override
     * @returns {string}
     */

  }, {
    key: 'getAfterCursor',
    value: function getAfterCursor() {
      var _getCursor2 = this.getCursor();

      var line = _getCursor2.line;
      var ch = _getCursor2.ch;

      var lines = this.getLines();
      var linesAfterCursor = lines.slice(line + 1);
      var currentLineAfterCursor = lines[line].slice(ch);
      return [currentLineAfterCursor].concat(linesAfterCursor).join(this.lineSeparator());
    }

    /**
     * @private
     * @returns {string[]}
     */

  }, {
    key: 'getLines',
    value: function getLines() {
      return this.cm.doc.getValue().split(this.lineSeparator());
    }

    /**
     * @private
     * @returns {{line: number, ch: number}}
     */

  }, {
    key: 'getCursor',
    value: function getCursor() {
      return this.cm.doc.getCursor();
    }

    /**
     * @private
     * @returns {string}
     */

  }, {
    key: 'lineSeparator',
    value: function lineSeparator() {
      return this.cm.doc.lineSeparator();
    }

    /**
     * @private
     * @param {CodeMirror} cm
     * @param {KeyboardEvent} e
     */

  }, {
    key: 'onKeydown',
    value: function onKeydown(cm, e) {
      var code = this.getCode(e);
      var event;
      switch (code) {
        case _editor.OTHER:
          return;
        case _editor.ENTER:
          {
            event = this.emitEnterEvent();
            break;
          }
        default:
          {
            event = this.emitMoveEvent(code);
          }
      }
      if (event.defaultPrevented) {
        e.preventDefault();
      }
    }

    /**
     * @private
     * @param {CodeMirror} cm
     * @param {KeyboardEvent} e
     */

  }, {
    key: 'onKeyup',
    value: function onKeyup(cm, e) {
      if (!this.isMoveKeyEvent(e)) {
        this.emitChangeEvent();
      }
    }

    /**
     * @private
     * @param {KeyboardEvent} e
     * @returns {boolean}
     */

  }, {
    key: 'isMoveKeyEvent',
    value: function isMoveKeyEvent(e) {
      var code = this.getCode(e);
      return code === _editor.DOWN || code === _editor.UP;
    }

    /**
     * @private
     */

  }, {
    key: 'startListening',
    value: function startListening() {
      this.cm.on('keydown', this.onKeydown);
      this.cm.on('keyup', this.onKeyup);
    }

    /**
     * @private
     */

  }, {
    key: 'stopListening',
    value: function stopListening() {
      this.cm.off('keydown', this.onKeydown);
      this.cm.off('keyup', this.onKeyup);
    }
  }]);

  return Codemirror;
}(_editor2.default);

exports.default = Codemirror;